#include "wx/wx.h"
#include "image_scope.h"
#include "image_object.h"

BEGIN_EVENT_TABLE (ImageScope, wxControl)
    EVT_PAINT(ImageScope::OnPaint)
    EVT_ERASE_BACKGROUND(ImageScope::OnEraseBackground)
    EVT_SET_FOCUS(ImageScope::OnSetFocus)
    EVT_KILL_FOCUS(ImageScope::OnKillFocus)
END_EVENT_TABLE()

ImageScope::~ImageScope()
{
    if ( mImageObj )
        mImageObj->Release();
}

ImageScope::ImageScope( wxWindow *parent_, wxWindowID id_, ImageObject *imgObj_,
      const wxPoint &pos_, const wxSize &size_,
      long style_, const wxString &name_ )
    : wxWindow(parent_,id_, pos_, size_, style_, name_)
{
    mImageObj = imgObj_;
    if ( mImageObj )
        mImageObj->AddRef();
    Enable(true);
}

void ImageScope::SetImageObject(ImageObject * imgObj_ )
{
    if ( mImageObj )
        mImageObj->Release();
    mImageObj = imgObj_;
    if ( mImageObj )
        mImageObj->AddRef();
    Refresh();
}

wxImage * ImageScope::GetImage()
{
    if ( !mImageObj )
        return NULL;
    return mImageObj->GetImage();
}

// TODO: speedup
// TODO: is this responsible for overdrawing the focus-border?
void ImageScope::OnPaint(wxPaintEvent& event)
{
    wxPaintDC dc(this);

    if ( mImageObj &&  mImageObj->GetImage())
    {
        wxMemoryDC memoryDC;
        
        wxBitmap *bmp = new wxBitmap(*mImageObj->GetImage());
        memoryDC.SelectObject(*bmp);
        
        int sizeX, sizeY;
        GetClientSize(&sizeX, &sizeY);
        double scaleX=(double)sizeX/(double)bmp->GetWidth();
        double scaleY=(double)sizeY/(double)bmp->GetHeight();
        dc.SetUserScale(scaleX, scaleY);
        
        dc.Blit(0,0, bmp->GetWidth(), bmp->GetHeight(), &memoryDC, 0, 0 );
       
        delete bmp;
        bmp = NULL;
    }
}

void ImageScope::OnEraseBackground(wxEraseEvent& event_)
{
    return;
}

void ImageScope::OnSetFocus(wxFocusEvent& event_)
{
    fprintf(stderr, "OnSetFocus\n");
    SetWindowStyleFlag( wxSIMPLE_BORDER );     // doesn't work :(
    Refresh();
}

void ImageScope::OnKillFocus(wxFocusEvent& event_)
{
    fprintf(stderr, "OnKillFocus\n");
    SetWindowStyleFlag( 0 );    // doesn't work :(
    Refresh();
}

